import xml.etree.ElementTree as et
import pandas as pd

def get_data_frame_from_XML_Nodes_List (nodes_list, df_cols): 
    rows = []
    for node in nodes_list: 
        res = []
        for el in df_cols[0:]: 
            #print(el)
            if node is not None and node.find(el) is not None:
                res.append(node.find(el).text)
            else: 
                res.append(None)
        rows.append({df_cols[i]: res[i] 
                     for i, _ in enumerate(df_cols)})
    
    out_df = pd.DataFrame(rows, columns=df_cols)
        
    return out_df


def get_leaf_node_names(elem):
    elemList = []

    for selem in elem.iter():
        #print(selem)
        if len(list(selem)) ==0:
            elemList.append(selem.tag)
    return elemList

def get_main_node_names(elem):
    elemList = []

    children = elem.getchildren()
    for child in children:
        if len(list(child)) ==0:
            elemList.append(child.tag)
    return elemList


import requests
from requests_toolbelt import MultipartEncoder

import inforion.ionapi.model.inforlogin as inforlogin
import logging as log
#from logger import get_logger
import json

#logger = get_logger("my_module")

def get_messaging_ping():
    try:
        url = inforlogin.base_url() + '/IONSERVICES/api/ion/messaging/service/ping'
        headers = inforlogin.header()
        res = requests.get(url, headers=headers)
        log.info('messaging ping: {}'.format(res.content))
        return res
    except Exception as e:
        log.error("Error ocurred " + str(e))



def post_messaging_v2_multipart_message(parameter_request, message_payload):
    try:
        url = inforlogin.base_url() + '/IONSERVICES/api/ion/messaging/service/v2/multipartMessage'
        data = MultipartEncoder(
            fields={'ParameterRequest': ('filename', json.dumps(parameter_request), 'application/json'),
                    'MessagePayload': ('filename', message_payload, 'application/octet-stream')
                    }
        )
        headers = inforlogin.header()
        headers.update({'Content-Type': data.content_type})

        res = requests.post(url, headers=headers, data=data)
        log.info('messaging v2 multipart message: {}'.format(res.content))
        return res
    except Exception as e:
        log.error("Error ocurred " + str(e))




import os
import sys

import pandas as pd


def checkfile_exists(file):
    if os.path.exists(file) is False:
        return False
    else:
        return True


def checkfiletype(filepath):

    # Split the extension from the path and normalise it to lowercase.
    ext = os.path.splitext(filepath)[-1].lower()

    if ext == ".csv" or ext == ".xlsx" or ext == ".xls":
        return ext
    else:
        print("Type is not supported")
        sys.exit(0)


def loadfile(file):

    ext = checkfiletype(file)

    if ext == ".csv":
        df = pd.read_csv(file)
    elif ext == ".xlsx" or ext == ".xls":
        df = pd.read_excel(file)
    return df


def savetodisk(file, df):

    ext = checkfiletype(file)

    if ext == ".csv":
        df.to_csv(file)
    else:

        writer = pd.ExcelWriter(file, engine="xlsxwriter")
        df.to_excel(writer, sheet_name="Log Output", index=False)
        writer.save()


def getDataFrame(inputfile):
    checkfiletype(inputfile)
    df = loadfile(inputfile)
    return df



import requests
import xml.etree.ElementTree as et
import pandas as pd

import inforion.ln.xml_helper as xhelper
import inforion.ln.soap_helper as shelper

class bcolors:
    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


available_services = [
    'BusinessPartner_v3',
    'SalesOrder'
    ]


def get_data(base_url, token, company, service_name):
    url = getRequestURL(base_url, service_name)

    request_body = shelper.get_list_request_xml(company, service_name)
    resp =  shelper.execute_soap_request(url, token, request_body)

    data = getDataFromResponse(resp, service_name)
    return data



def export_data(url, token, company, service_name, outputfile):

    if(service_name == 'BusinessPartner'):
        service_name ==  'BusinessPartner_v3'

    if service_name not in available_services:
        available_services_str = ', '.join(available_services)
        print(bcolors.FAIL + f'{service_name} is currently not available. Currently we only support {available_services_str}.'+bcolors.ENDC)
        return

    data = get_data(url, token, company, service_name)
    if len(data) > 0:
        df_cols = xhelper.get_main_node_names(data[0])
        df = xhelper.get_data_frame_from_XML_Nodes_List(data, df_cols)
        df.to_excel(outputfile)
        print(bcolors.OKGREEN + "Exported data successfully to Excel file. "+bcolors.ENDC)
    else:
        print(bcolors.FAIL + f'No data found to export.'+bcolors.ENDC)


def getDataFromResponse(resp, service_name):
    xroot = et.fromstring(resp.content)
    rows = []

    body = xroot[0]
    if isSuccess(body):
        listresp = body[0]
        listresp2 = listresp[0]
        data = listresp2[0]

        for partner in data.findall(service_name):
            rows.append(partner)
    else:
        errorMessage = getExceptionMessage(body)
        print(bcolors.FAIL +  errorMessage +bcolors.ENDC)

    return rows


def getRequestURL(url, method):
    if url[-1] != '/':
        url = url + '/'
   
    if url.endswith(method):
        return url

    return url + method


def isSuccess(body):
    child = body[0]
    if 'Fault' in child.tag:
        return False
    return True

def getExceptionMessage(body):
    child = body[0]
    detail = child[2]
    detailMessage = detail[1]
    return detailMessage.text



# © 2017 Daniel Jordan: github.com/dajor
#
# This file is part of InforION.
#

#
# InforION is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with OCRmyPDF.  If not, see <http://www.gnu.org/licenses/>.


import pkg_resources

PROGRAM_NAME = 'inforion'

# Official PEP 396
__version__ = pkg_resources.get_distribution('inforion').version


# import setuptools


from __future__ import print_function, unicode_literals

from setuptools import find_packages, setup


import sys


def parse_requirements(filename):
    """ load requirements from a pip requirements file """
    lineiter = (line.strip() for line in open(filename))
    return [line for line in lineiter if line and not line.startswith("#")]


def load_requirements(fname):
    reqs = parse_requirements(fname)
    return [str(ir.req) for ir in reqs]


if sys.version_info < (3, 7):
    print("Python 3.7 or newer is required", file=sys.stderr)
    sys.exit(1)

# pylint: disable=w0613

command = next((arg for arg in sys.argv[1:] if not arg.startswith("-")), "")
if command.startswith("install") or command in [
    "check",
    "test",
    "nosetests",
    "easy_install",
]:
    forced = "--force" in sys.argv
    if forced:
        print("The argument --force is deprecated. Please discontinue use.")


if "upload" in sys.argv[1:]:
    print("Use twine to upload the package - setup.py upload is insecure")
    sys.exit(1)


tests_require = open("requirements/test.txt", encoding="utf-8").read().splitlines()


def readme():
    with open("README.md", encoding="utf-8") as f:
        return f.read()


with open("README.md", "r") as fh:
    long_description = fh.read()

install_reqs = parse_requirements("requirements/main.txt")
reqs = install_reqs


setup(
    name="inforion",  # Replace with your own username
    version_format="{tag}.{commitcount}",
    setup_requires=["setuptools-git-version"],
    author="Daniel Jordan",
    author_email="daniel.jordan@fellow-consulting.de",
    description="Infor ION Package for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Fellow-Consulting-AG/inforion/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    packages=find_packages("src", exclude=["tests", "tests.*"]),
    package_dir={"": "src"},
    entry_points={"console_scripts": ["inforion = inforion.__main__:main"]},
    keywords=["Infor", "InforION", "Datalake", "LN", "M3"],
    install_requires=reqs,
    zip_safe=True,
    include_package_data=True,
    package_data={
        "inforion": [
            "ionapi/controller/*",
            "ionapi/model/*",
            "ionapi/*",
            "helper/*",
            "transformation/*",
            "excelexport/*",
            "excelexport/api-files/*.json",
        ]
    },
    data_dir=[(("config", ["excelexport/api-files/"]))],
)



#!/usr/bin/env python3
import logging
import os.path

import click
import inforion as infor
from inforion.datacatalog.datacatalog import delete_datacatalog_object
from inforion.datacatalog.datacatalog import ObjectSchemaType
from inforion.datacatalog.datacatalog import post_datacatalog_object
from inforion.datalake.datalake import delete_v1_purge_filter
from inforion.datalake.datalake import delete_v1_purge_id
from inforion.datalake.datalake import get_v1_payloads_list
from inforion.datalake.datalake import get_v1_payloads_stream_by_id
from inforion.excelexport import *
from inforion.helper.filehandling import *
from inforion.ionapi.controller import *
from inforion.ionapi.model import *
import inforion.ln as lni

import logging
import os.path
import os

import logging
#from logger import get_logger

from inforion.datacatalog.datacatalog import (
    post_datacatalog_object,
    delete_datacatalog_object,
    ObjectSchemaType,
)
from inforion.messaging.messaging import post_messaging_v2_multipart_message
from inforion.ionapi.model import inforlogin
from inforion.messaging.messaging import post_messaging_v2_multipart_message

# TODO update to use multi modules log
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger("main")


@click.group()
def main():
    """Generell section\n
    Please see the dodcumentation on https://inforion.readthedocs.io/
    """

    pass


@click.command(
    name="load",
    help="Section to load data to Infor ION. Right now we support Excel and CSV Data to load",
)
@click.option(
    "--url",
    "-u",
    required=True,
    prompt="Please enter the url",
    help="The full URL to the API is needed. Please note you need to enter the full url like .../M3/m3api-rest/v2/execute/CRS610MI",
)
@click.option(
    "--ionfile",
    "-f",
    required=True,
    prompt="Please enter the location ionfile",
    help="IONFile is needed to login in to Infor OS. Please go into ION and generate a IONFile. If not provided, a prompt will allow you to type the input text.",
)
@click.option(
    "--program",
    "-p",
    required=True,
    prompt="Please enter Program",
    help="What kind of program to use by the load",
)
@click.option(
    "--method",
    "-m",
    required=True,
    prompt="Please enter the method",
    help="Select the method as a list",
)
@click.option(
    "--inputfile",
    "-i",
    required=True,
    prompt="Please enter the InputFile",
    help="File to load the data. Please use XLSX or CSV format. If not provided, the input text will just be printed",
)
@click.option(
    "--outputfile", "-o", help="File as Output File - Data are saved here for the load"
)
@click.option(
    "--start", "-s", type=int, help="Dataload can be started by 0 or by a number"
)
@click.option("--end", "-e", type=int, help="Dataload can be end")
@click.option("--configfile", "-z", help="Use a Configfile instead of parameters")
def load(
    url,
    ionfile,
    program,
    method,
    inputfile,
    outputfile,
    configfile,
    start=None,
    end=None,
):

    if checkfile_exists(inputfile) is False:
        click.secho("Error:", fg="red", nl=True)
        click.echo("Inputfile does not exist")
        sys.exit(0)

    if configfile is not None:

        with open(configfile) as file:
            config_json = json.load(file)

            if all(
                k in config_json
                for k in ("url", "ionfile", "program", "method", "inputfile")
            ):
                url = config_json["url"]
                ionfile = config_json["ionfile"]
                program = config_json["program"]
                method = config_json["method"]
                inputfile = config_json["inputfile"]
                outputfile = config_json["outputfile"]
            else:
                logging.error("JSON File wrong config")
                sys.exit(0)
            if "start" in config_json:
                start = config_json["start"]
            else:
                start = None

            if "end" in config_json:
                end = config_json["end"]
            else:
                end = None

    dataframe = pd.read_excel(inputfile, dtype=str)

    return infor.main_load(
        url, ionfile, program, method, dataframe, outputfile, start, end
    )


@click.command(name="extract", help="Section to generate empty mapping sheets")
@click.option("--program", "-p", help="Choose the program to extract the sheets from")
@click.option(
    "--outputfile", "-o", help="File as Output File - Data are saved here for the load"
)
def extract(program, outputfile):

    if not "program" in locals() or not program:
        logging.info("\033[91m" + "Error: Program name is missing" + "\033[0m")
    if not "outputfile" in locals() or not outputfile:
        logging.info("\033[91m" + "Error: Output filename is missing" + "\033[0m")

    if program and outputfile:
        generate_api_template_file(program, outputfile)


@click.command(name="transform", help="section to do the transformation")
@click.option("--mappingfile", "-a", help="Please define the Mapping file")
@click.option("--mainsheet", "-b", help="Please define the mainsheet")
@click.option(
    "--inputfile",
    "-i",
    help="File to load the data. Please use XLSX or CSV format. If not provided, the input text will just be printed",
)
@click.option(
    "--outputfile", "-o", help="File as Output File - Data are saved here for the load"
)
def transform(mappingfile, mainsheet, inputfile, outputfile):
    inputdata = pd.read_excel(inputfile)
    return infor.main_transformation(mappingfile, mainsheet, inputdata, outputfile)


@main.group(name="catalog")
def catalog():
    """Commands related to Data Catalog."""
    pass


@main.group(name="datalake")
def datalake():
    """Commands related to Data Lake."""
    pass


@catalog.command(name="create", help="Catalog create")
@click.option("--ionfile", "-i", help="Please define the ionapi file")
@click.option("--name", "-n", help="Please define the object name")
@click.option("--schema_type", "-t", help="Please define the object schema type")
@click.option("--schema", "-s", help="Please define the schema file")
@click.option("--properties", "-p", help="Please define the schema properties file")
def create(ionfile, name, schema_type, schema, properties):
    inforlogin.load_config(ionfile)
    inforlogin.login()

    if not os.path.isfile(schema):
        raise FileNotFoundError("Schema file not found.")

    if not os.path.isfile(properties):
        raise FileNotFoundError("Properties file not found.")

    with open(schema, "r") as file:
        schema_content = json.loads(file.read())
    with open(properties, "r") as file:
        properties_content = json.loads(file.read())
    response = post_datacatalog_object(
        name, ObjectSchemaType(schema_type), schema_content, properties_content
    )

    if response.status_code == 200:
        logger.info("Data catalog schema {} was created.".format(name))
    else:
        logger.error(response.content)


@catalog.command(name="delete", help="Catalog delete")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--name", "-n", help="Please define the object name")
def delete(ionfile, name):
    inforlogin.load_config(ionfile)
    inforlogin.login()
    response = delete_datacatalog_object(name)

    if response.status_code == 200:
        logger.info("Data catalog schema {} was deleted.".format(name))
    else:
        logger.error(response.content)


@datalake.command(name="upload", help="Datalake upload")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--schema", "-s", help="Please define the schema name")
@click.option("--logical_id", "-l", help="Please define the fromLogicalId")
@click.option("--file", "-f", help="Please define the file")
def upload(ionfile, schema, logical_id, file):
    inforlogin.load_config(ionfile)
    inforlogin.login()
    parameter_request = {
        "documentName": schema,
        "fromLogicalId": logical_id,
        "toLogicalId": "lid://default",
        "encoding": "NONE",
        "characterSet": "UTF-8",
    }
    with open(file, "rb") as file:
        message_payload = file.read()
    response = post_messaging_v2_multipart_message(parameter_request, message_payload)

    if response.status_code == 201:
        logger.info("Document uploaded successfully.")
    else:
        logger.error(response.content)


@datalake.command(name="list", help="Datalake list")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--list_filter", "-f", help="Please define the filter")
@click.option("--sort", "-s", help="Please define the sort")
@click.option("--page", "-p", help="Please define the page")
@click.option("--records", "-r", help="Please define the records")
def datalake_list(ionfile, list_filter=None, sort=None, page=None, records=None):
    """
    List data object properties using a filter.
    :param ionfile: Infor IONAPI credentials file.
    :param list_filter: The restrictions to be applied on the returned records.
    :param sort: Field name followed by colon followed by direction (asc or desc; default asc).
    Example: 'event_date:desc'.
    :param page: The page number from which to start returning records. Starts from 1.
    :param records: The number of records that will be returned. Starts from 0
    """
    inforlogin.load_config(ionfile)
    inforlogin.login()
    response = get_v1_payloads_list(list_filter, sort, page, records)

    if response.status_code == 200:
        click.echo(response.text)
    else:
        logger.error(response.content)


@datalake.command(name="purge", help="Datalake purge")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--ids", "-id", help="Please define the ids")
@click.option("--purge_filter", "-f", help="Please define the filter")
def datalake_purge(ionfile, ids, purge_filter):
    """
    Deletes Data Objects based on the given filter or a list of IDs.
    You cannot define both arguments: ids and purge_filter.
    :param ionfile: Infor IONAPI credentials file.
    :param ids: Object ids.
    :param purge_filter: The restrictions to be applied to purge the records
    """
    if ids is not None and purge_filter is not None:
        raise ValueError("You cannot define both arguments: ids and purge_filter.")

    inforlogin.load_config(ionfile)
    inforlogin.login()

    if ids is not None:
        ids_list = ids.split(",")
        response = delete_v1_purge_id(ids_list)
        if response.status_code == 200:
            click.echo(response.text)
        else:
            logger.error(response.content)

    if purge_filter is not None:
        response = delete_v1_purge_filter(purge_filter)
        if response.status_code == 200:
            click.echo(response.text)
        else:
            logger.error(response.content)


@datalake.command(name="get", help="Datalake get")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--stream_id", "-id", help="Please define the id")
def datalake_get(ionfile, stream_id):
    """
    Retrieve payload based on id from datalake.
    :param ionfile: Infor IONAPI credentials file.
    :param stream_id: Object ID.
    """
    inforlogin.load_config(ionfile)
    inforlogin.login()
    response = get_v1_payloads_stream_by_id(stream_id)

    if response.status_code == 200:
        click.echo(response.text)
    else:
        logger.error(response.content)


@main.group(name="check")
def check():
    """Commands to check soemthing"""
    pass


@check.command(name="Check_Token", help="Check Login Token")
@click.option("--url", "-u", help="URL to local ION")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
def check_token(url, ionfile):
    """Check Login and display the token"""
    inforlogin.load_config(ionfile)
    inforlogin.login()


@main.group(name="ln")
def ln():
    """Commands related to Export Data from LN."""
    pass


@ln.command(name="ExportData", help="Export Data")
@click.option("--url", "-u", help="URL to local ION")
@click.option("--ionfile", "-i", help="Please define the ionfile file")
@click.option("--company", "-c", help="Company for which we need to export")
@click.option("--service_name", "-s", help="Service name. e.g. BusinessPartner, SalesOrder")
@click.option("--outputfile", "-o", help="File as Output File")
def export_data(url, ionfile, company, service_name, outputfile):
    """Exports business partner to an Excel file"""

    inforlogin.load_config(ionfile)
    token = inforlogin.login()['access_token']
    lni.export_data(url, token, company, service_name, outputfile)

main.add_command(load)
main.add_command(transform)
main.add_command(extract)
main.add_command(check)

if __name__ == "__main__":
    main()




import requests
import xml.etree.ElementTree as et

import inforion.ln.xml_helper as xhelper

def execute_soap_request(url, token, request_body):
    headers = {'content-type': 'text/xml',
                'Authorization': 'Bearer '+token}

    response = requests.post(url,data=request_body,headers=headers)
    return response


def get_base_request_xml(company, service_name):
    xml_string = f"""<?xml version="1.0" encoding="UTF-8"?>
            <Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
                <Header>
                    <Activation xmlns="http://www.infor.com/businessinterface/{service_name}">
                        <company xmlns="">{company}</company>
                    </Activation>
                </Header>
                <Body>
                [[body]]
                </Body>
            </Envelope>"""

    return xml_string


def get_method_xml(service_name, method_name):
    if method_name == 'List':
        return f"""<List xmlns="http://www.infor.com/businessinterface/{service_name}">
                        <ListRequest></ListRequest>
                    </List>"""
    

def get_list_request_xml(company, service_name):
    method_name = 'List'
    xml = get_base_request_xml(company, method_name)
    method_xml = get_method_xml(service_name, method_name)
    return xml.replace('[[body]]', method_xml)